@@ -21,6 +21,7 @@
 #define	private	static
 #define	public
 
+#ifndef OS2
 extern	int	atoi();
 extern	void	exit();
 extern	char	*strcpy(), *strcat();
@@ -30,6 +31,7 @@ extern	double	atof(const char *);
 #else
 extern	double	atof();
 #endif
+#endif  /* OS2 */
 
 #define SWAP( TYPE, A, B )		\
   {					\
@@ -40,7 +42,7 @@ extern	double	atof();
     (B) = TMP;				\
   }					\
 
-
+#ifndef OS2
 #ifdef SYS_V
 #    define	bcopy( A, B, C )	memcpy( B, A, C )
 #    define	bcmp( A, B, C )		memcmp( B, A, C )
@@ -48,4 +50,4 @@ extern	void	memcopy();
 #else
 extern	void	bcopy();
 #endif
-
+#endif  /* OS2 */
