@@ -13,6 +13,12 @@
  */
 
 #include <stdio.h>
+#ifdef OS2
+#include <stdlib.h>
+#include <string.h>
+#include <time.h>
+#include <io.h>
+#endif  /* OS2 */
 #include "defs.h"
 #include "net.h"
 #include "globals.h"
@@ -73,7 +79,9 @@ private	Bits	*blist = NULL;		/* list of vectors */
 private	sptr	slist = NULL;		/* list of sequences */
 private	int	maxsequence = 0;	/* longest sequence defined */
 
-private	sptr	clock = NULL;		/* vectors which make up clock */
+
+public  int     contline = 0;
+private	sptr	xclock = NULL;		/* vectors which make up clock */
 private	int	maxclock = 0;		/* longest clock sequence defined */
 
 private	iptr	wlist = NULL;		/* list of nodes to be displayed */
@@ -104,7 +112,7 @@ public	FILE	*logfile = NULL;	/* log file of transactions */
 
 #ifdef	POWER_EST
 public	FILE	*caplogfile = NULL;	/* log file of cap transitions */
-public	float	toggled_cap = 0;	/* indicative of total power of chip */
+public	double	toggled_cap = 0;	/* indicative of total power of chip */
 public  float   vsupply = 5.0;		/* supply voltage for pwr estimation */
 public  float   capstarttime = 0.0;
 public  float   capstoptime = 0.0;
@@ -291,6 +299,8 @@ private int expand( string, buffer, bufsize, wc )
   }
 
 
+private	int applyStart = 1;
+
 /*
  * Apply given function to each argument on the command line.
  * Arguments are checked first to ensure they are the name of a node or
@@ -313,7 +323,7 @@ private void apply( fun, vfun, arg )
     register bptr  b;
     register int   i, j, found;
 
-    for( i = 1; i < targc; i += 1 )
+    for( i = applyStart ; i < targc; i += 1 )
       {
 	p = targv[i];
 	if( arg == plus_minus )
@@ -503,7 +513,7 @@ private int input( in )
 		(void) fputs( line, logfile );
 	  }
 	else if( dcmdfile )
-	    (void) fprintf( stdout, "%s> %s", filename, line );
+	    (void) fprintf( stdout, "%s:%d> %s", filename, lineno, line );
 
 	    /* convert line into tokens */
 	lineno += 1;
@@ -714,7 +724,7 @@ private void prtime( col )
   {
     if( col != 0 )
 	lprintf( stdout, "\n" );
-    lprintf( stdout, "time = %.1fns", d2ns( cur_delta ) );
+    lprintf( stdout, "time = %.2fns", d2ns( cur_delta ) );
     if( npending )
 	lprintf( stdout, "; there are pending events (%d)",npending );
     lprintf( stdout, "\n" );
@@ -1058,7 +1068,7 @@ private int dovector()
 	if( strcmp( b->name, targv[1] ) == 0 )
 	  {
 	    if( undefseq( (nptr) b, &slist, &maxsequence ) or
-	      undefseq( (nptr) b, &clock, &maxclock ) )
+	      undefseq( (nptr) b, &xclock, &maxclock ) )
 	      {
 		error( filename, lineno,
 		  "%s is a clock/sequence; can't change it while stoped\n",
@@ -1154,7 +1164,7 @@ private int setvector()
   }
 
 
-private void CompareVector( np, name, nbits, mask, value )
+private int CompareVector( np, name, nbits, mask, value )
   nptr  *np;
   char  *name;
   int   nbits;
@@ -1167,12 +1177,12 @@ private void CompareVector( np, name, nbits, mask, value )
     if( strlen( value ) != nbits )
       {
 	error( filename, lineno, "wrong number of bits for value\n" );
-	return;
+	return 0;
       }
     if( mask != NULL and strlen( mask ) != nbits )
       {
 	error( filename, lineno, "wrong number of bits for mask\n" );
-	return;
+	return 0;
       }
 
     for( i = 0; i < nbits; i++ )
@@ -1181,14 +1191,17 @@ private void CompareVector( np, name, nbits, mask, value )
 	    continue;
 	n = np[i];
 	if( (val = ch2pot( value[i] )) >= N_POTS )
-	    return;
+	    return 0;
 	if( val == X_X ) val = X;
 	if( n->npot != val )
 	    goto fail;
       }
-    return;
+    return 0;
 
   fail :
+#ifndef OLD_ASSERT
+    return 1;
+#else
     lprintf( stdout, "(%s, %d): assertion failed on '%s' ",
      filename, lineno, name );
     for( i = 0; i < nbits; i++ )
@@ -1202,6 +1215,7 @@ private void CompareVector( np, name, nbits, mask, value )
 	    lprintf( stdout, "%c", vchars[ np[i]->npot ] );
       }
     lprintf( stdout, " (%s)\n", value );
+#endif
   }
 
 
@@ -1244,8 +1258,10 @@ private void FindOne( f )
   }
   
 
+
+#ifdef OLD_ASSERT
 /* assert a bit vector */
-private int doAssert()
+public int doAssert()
   {
     char      *mask, *value, *name;
     Find1Arg  f;
@@ -1277,8 +1293,199 @@ private int doAssert()
 
     return( 0 );
   }
+#endif
 
 
+/* assert a bit vector */
+private int doAssert()
+  {
+    char      *mask, *value, *name;
+    Find1Arg  f;
+    int i, nbits, comp = 0;
+    nptr *nodes;
+
+    if( targc == 4 )
+      {
+	mask = targv[2];
+	value = targv[3];
+      }
+    else
+      {
+	mask = NULL;
+	value = targv[2];
+      }
+
+    FindOne( &f );
+
+    if( f.num > 1 )
+	error( filename, lineno, "%s matches more than one node or vector\n",
+	  targv[1] );
+    else if( f.node != NULL )
+      {
+	name = pnode( f.node );
+	UnAlias( f.node );
+	comp = CompareVector( &f.node, name, 1, mask, value );
+        nodes = &f.node;
+        nbits = 1;
+      }
+    else if( f.vec != NULL ) { 
+	comp = CompareVector( f.vec->nodes, f.vec->name, f.vec->nbits, mask, value );
+        name = f.vec->name;
+        nbits = f.vec->nbits;
+        nodes = f.vec->nodes; 
+    }
+    if(comp != 0) {
+        lprintf( stdout, "(%s, %d): assertion failed on '%s' ",
+         filename, lineno, name );
+        for( i = 0; i < nbits; i++ )
+          {
+     	    if( mask != NULL and mask[i] != '0' )
+	      {
+	       lprintf( stdout, "-" );
+	       value[i] = '-';
+	      }
+	   else
+	       lprintf( stdout, "%c", vchars[ nodes[i]->npot ] );
+          }
+        lprintf( stdout, " (%s)\n", value );
+        }
+    return( 0 );
+  }
+
+
+
+private int doUntil( )
+  {
+    char      *mask, *value, *name;
+    Find1Arg  f;
+    int i, nbits, ccount, cnt = 0, comp = 0;
+    nptr *nodes;
+
+    if( targc == 5 )
+      {
+	mask = targv[2];
+	value = targv[3];
+        ccount = atoi(targv[4]);
+      }
+    else
+      {
+	mask = NULL;
+	value = targv[2];
+        ccount = atoi(targv[3]);
+      }
+
+    FindOne( &f );
+
+    if( f.num > 1 )
+	error( filename, lineno, "%s matches more than one node or vector\n",
+	  targv[1] );
+    else if( f.node != NULL )
+      {
+	name = pnode( f.node );
+	UnAlias( f.node );
+        targc = 1;
+	while((cnt <= ccount) && (comp = CompareVector( &f.node, name, 1, mask, value ) != 0)) {
+            cnt++;
+            clockit(1); }
+        nodes = &f.node;
+        nbits = 1;
+      }
+    else if( f.vec != NULL ) {
+        targc = 1;
+	while((cnt <= ccount) && (comp = CompareVector( f.vec->nodes, f.vec->name, f.vec->nbits, mask,
+            value ) != 0)) {
+            cnt++;
+            clockit(1); }
+        name = f.vec->name;
+        nbits = f.vec->nbits;
+        nodes = f.vec->nodes; }
+    if(comp != 0) {
+        lprintf( stdout, "(%s, %d): assertion failed on '%s' ",
+         filename, lineno, name );
+        for( i = 0; i < nbits; i++ )
+          {
+     	    if( mask != NULL and mask[i] != '0' )
+	      {
+	       lprintf( stdout, "-" );
+	       value[i] = '-';
+	      }
+	   else
+	       lprintf( stdout, "%c", vchars[ nodes[i]->npot ] );
+          }
+        lprintf( stdout, " (%s)\n", value );
+        }
+    return( 0 );
+  }
+
+
+private	nptr aw_trig ; /* keeps current assertWhen trigger */
+private awptr aw_p ;   /* track pointer on the current assertWhen list */
+
+private setupAssertWhen(n, val)
+ nptr n;
+ char *val;
+  {
+	register awptr p;
+
+	p = ( awptr ) Falloc(sizeof(assertWhen),1);
+	p->node = n ;
+	p->val  = *val ;
+	p->nxt = NULL ;
+
+	if ( aw_trig->awpending == NULL )  { /* first time */
+		aw_trig->awpending = p ;
+		aw_p = p ;
+	}
+	else  { /* more than 1 matching nodes */
+		aw_p->nxt = p ;
+		aw_p = p ;
+	}
+	return 1;
+  }
+
+private int doAssertWhen()
+{
+    Find1Arg  trig;
+
+    FindOne( &trig );
+
+    if( trig.num > 1 )
+	error( filename, lineno, "%s matches more than one node or vector\n",
+	  targv[1] );
+    else if( trig.node != NULL )
+      {
+	applyStart = 3; targc  = 4;
+
+	UnAlias( trig.node );
+	aw_trig = trig.node ;
+	aw_trig->awpot = ch2pot( *targv[2] );
+	apply(setupAssertWhen, NULL, targv[4]);
+
+	applyStart = 1; targc = 4;
+      }
+    else if( trig.vec != NULL ) 
+	error( filename, lineno, "trigger to assertWhen %s can't be a vector\n",
+		targv[1]);
+    return( 0 );
+}
+
+public	void	evalAssertWhen(n)
+  nptr n;
+{
+	register awptr p, p2 ; 
+	char cval[2] ; 
+
+	cval[0] = 0 ;
+	cval[1] = 0 ;
+	for ( p = n->awpending ; p ;  ) {
+		cval[0] = p->val ; 
+		CompareVector(&(p->node), pnode(p->node), 1, NULL, cval);
+		p2 = p ; p = p->nxt ;
+		Ffree(p2, sizeof( assertWhen ) );
+	}
+	n->awpending = NULL ;
+}
+
 private int collect_inputs( n, inps )
   register nptr  n;
   register nptr  inps[];
@@ -1352,7 +1559,7 @@ public void disp_watch_vec( which )
     char           temp[20];
 
     which &= (WATCHVECTOR | STOPVECCHANGE);
-    (void) sprintf( temp, " @ %.1fns ", d2ns( cur_delta ) );
+    (void) sprintf( temp, " @ %.2fns ", d2ns( cur_delta ) );
     lprintf( stdout, "%s", temp );
     column = strlen( temp );
     for( b = blist; b != NULL; b = b->next )
@@ -1672,7 +1879,7 @@ public void rm_del_from_lists()
     vec_del = mark_deleted_vectors();
 
     maxsequence = rm_from_seq( &slist );
-    maxclock = rm_from_seq( &clock );
+    maxclock = rm_from_seq( &xclock );
 
     if( analyzerON ) RemoveAllDeleted();
 
@@ -1759,7 +1966,7 @@ private int setclock()
   {
     CHECK_STOP();
 	/* process sequence and add to clock list */
-    defsequence( &clock, &maxclock );
+    defsequence( &xclock, &maxclock );
     return( 0 );
   }
 
@@ -1771,7 +1978,7 @@ private int step_phase()
   {
     static int  which_phase = 0;
 
-    vecvalue( clock, which_phase );
+    vecvalue( xclock, which_phase );
     if( relax( cur_delta + stepsize ) )
 	return( 1 );
     which_phase = (which_phase + 1) % maxclock;
@@ -1800,7 +2007,7 @@ private int clockit( n )
   {
     register int  i;
 
-    if( clock == NULL )
+    if( xclock == NULL )
 	error( filename, lineno, "no clock nodes defined!\n" );
     else
       {
@@ -2162,7 +2369,7 @@ private int setdecay()
 	if( tdecay == 0 )
 	    lprintf( stdout, "decay = No decay\n" );
 	else
-	    lprintf( stdout, "decay = %.1fns\n", d2ns( tdecay ) );
+	    lprintf( stdout, "decay = %.2fns\n", d2ns( tdecay ) );
       }
     else
       {
@@ -2183,7 +2390,7 @@ private int setunit()
 	if( tunitdelay == 0 )
 	    lprintf( stdout, "unitdelay = OFF\n" );
 	else
-	    lprintf( stdout, "unitdelay = %.1f\n", d2ns( tunitdelay ) );
+	    lprintf( stdout, "unitdelay = %.2f\n", d2ns( tunitdelay ) );
       }
     else
       {
@@ -2232,7 +2439,7 @@ private void cpath( n, level )
 	 */
     else if( n->t.cause == n )
       {
-	lprintf( stdout, "  %s -> %c @ %.1fns , node was an input\n",
+	lprintf( stdout, "  %s -> %c @ %.2fns , node was an input\n",
 	  pnode( n ), vchars[ n->npot ], d2ns( n->c.time ) );
       }
     else if( n->t.cause->nflags & VISITED )
@@ -2248,10 +2455,10 @@ private void cpath( n, level )
 	cpath( n->t.cause, level + 1 );
 	n->nflags &= ~VISITED;
 	if( delta_t < 0 )
-	    lprintf( stdout, "  %s -> %c @ %.1fns   (??)\n", pnode( n ),
+	    lprintf( stdout, "  %s -> %c @ %.2fns   (??)\n", pnode( n ),
 		vchars[ n->npot ], d2ns( n->c.time ) );
 	else
-	    lprintf( stdout, "  %s -> %c @ %.1fns   (%.1fns)\n", pnode( n ),
+	    lprintf( stdout, "  %s -> %c @ %.2fns   (%.2fns)\n", pnode( n ),
 		vchars[ n->npot ], d2ns( n->c.time ), d2ns( delta_t ) );
       }
   }
@@ -2341,11 +2548,11 @@ private int doactivity()
     for( total = i = 0; i < NBUCKETS; i++ ) total += ac.table[i];
 
     lprintf( stdout,
-      "Histogram of circuit activity: %.1f -> %.1fns (bucket size = %.1f)\n",
+      "Histogram of circuit activity: %.2f -> %.2fns (bucket size = %.2f)\n",
       d2ns( ac.begin ), d2ns( ac.end ), d2ns( ac.size ) );
 
     for( i = 0; i < NBUCKETS; i += 1 )
-	lprintf( stdout, " %10.1f -%10.1f%6d  %s\n",
+	lprintf( stdout, " %10.2f -%10.2f%6d  %s\n",
 	  d2ns(ac.begin + (i * ac.size)), d2ns(ac.begin + (i + 1) * ac.size),
 	  ac.table[i], &st[ SIZE_ST - (SIZE_ST * ac.table[i]) / total ] );
 
@@ -2401,7 +2608,7 @@ private int dochanges()
       }
 
     column = 0;
-    lprintf(stdout,"Nodes with last transition in interval %.1f -> %.1fns:\n",
+    lprintf(stdout,"Nodes with last transition in interval %.2f -> %.2fns:\n",
       d2ns( ac.begin ), d2ns( ac.end ) );
 
     walk_net( cdoit, (char *) &ac );
@@ -2498,7 +2705,7 @@ private int print_list( n, l, eolist )
 	return( n );
     for( eolist = eolist->flink; l != eolist and n != 0; l = l->flink, n-- )
       {
-	lprintf( stdout, "Node %s -> %c @ %.1fns (%.1fns)\n",
+	lprintf( stdout, "Node %s -> %c @ %.2fns (%.2fns)\n",
 	  pnode( l->enode ), vchars[ l->eval ], d2ns( l->ntime ),
 	  d2ns( l->ntime - cur_delta ) );
       }
@@ -2589,7 +2796,7 @@ private int print_tcap()
 private int set_incres()
   {
     if( targc == 1 )
-	lprintf( stdout, "incremental resolution = %.1f\n", d2ns(INC_RES) );
+	lprintf( stdout, "incremental resolution = %.2f\n", d2ns(INC_RES) );
     else
       {
 	long  new_res = (long) ns2d( atof( targv[1] ) );
@@ -2687,7 +2894,11 @@ private char    x_display[40];
 
 private int xDisplay()
   {
+#ifndef OS2
     char  *s, *getenv();
+#else
+    char *s;
+#endif
 
     if( targc == 1 )
       {
@@ -3188,6 +3399,19 @@ private int do_time()
     return( i );
   }
 
+private int doHist()
+  {
+     if(targc == 1) {
+        lprintf( stdout, "History is ");
+        if(sm_stat && OUT_OF_MEM) lprintf( stdout, "off.\n");
+        else lprintf( stdout, "on.\n"); }
+     else {
+        if(strcmp(targv[1], "on") != 0) {
+           sm_stat |= OUT_OF_MEM; }
+        else { sm_stat &= ~OUT_OF_MEM; }
+     }
+  return (0);
+  }
 
 /*
  * Flush out the recorded history up to the (optional) specified time.
@@ -3478,6 +3702,8 @@ private Command  cmds[] =
       "[-b|-o|-h] node/vector... -> display node/vector(s) in analyzer"	},
     { "assert",		doAssert,	3,	4,
       "node/vector [mask] val -> assert node/vector = val [& mask = 0]"	},
+    { "assertWhen",		doAssertWhen,	5,	5,
+      "nodeT valT node val -> assert node = val when nodeT switches to valT"},
     { "back",		back_time,	2,	2,
       "time -> move simulation time back to specified to time"		},
     { "c",		doclock,	1,	2,
@@ -3519,6 +3745,8 @@ private Command  cmds[] =
 #endif CL_STATS
     { "help",		do_help,	1,	MAXARGS,
       "[command]... -> print info on command(s) or available commands"	},
+    { "hist",           doHist,         1,      2,
+       "[on/off] -> display or set history collection mode"             },
     { "inputs",		inputs,		1,	1,
       " -> print currently driven (input) nodes"			},
     { "ires",		set_incres,	1,	2,
@@ -3573,6 +3801,8 @@ private Command  cmds[] =
       "[node/vector]... -> drive a node/vector(s) to X (undefined)"	},
     { "unitdelay",	setunit,	1,	2,
       "[time] -> force transitions to specified time (0 to disable)"	},
+    { "until",          doUntil,        4,      5,
+      "node/vec [mask] val count -> sim until = val [& mask = 0] or count runout"	},
     { "V",		setseq,		1,	MAXARGS,
       "[node/vector [val...]] -> define input sequence for node/vector"	},
     { "vector",		dovector,	3,	MAXARGS,
@@ -3660,6 +3890,9 @@ private void Usage( msg, s1 )
   }
 
 
+#ifdef OS2
+private int _Optlink cmd_cmp(Command *a, Command *b);
+#endif  /* OS2 */
 private int cmd_cmp( a, b )
   Command *a, *b;
   {
